 /*************************************************************************
 * FILE NAME: $RCSfile: fpm_example.c,v $    COPYRIGHT (c) FREESCALE 2004 *
 * DESCRIPTION:                                     All Rights Reserved   *
 * This is a sample MPC5500 program to show a simple use of the eTPU FPM  *
 * function. The ETPU QOM function is used to provide a stumulus for the  *
 * FPM function to measure. An external hardware connection is required   *
 * between the QOM channel and the FPM channel.                           *
 * The FPM function and API is used in continuous mode to count the number*
 * of rising edges in the QOM stimulus over 700 TCR1 clocks and measure   *
 * frequency in that window.                                              *
 *========================================================================*
 * ORIGINAL AUTHOR: Geoff Emerson (r47354)                                *
 *........................................................................*
 * 0.1   G.Emerson    27/Apr/04    Initial version.                       *
 * 0.2                             Updated for new build structure.       *
/**************************************************************************/
#include "mpc5554.h"            /* mpc5554 register definitions */
#include "mpc5554_vars.h"       /* Variables specific to mpc5554 */
#include "etpu_util.h"          /* useful utility routines */
#include "fs_gpio.h"            /* GPIO API for the MPC5500 family */
#include "etpu_set1.h"          /* eTPU standard function set 1 */
#include "etpu_qom.h"           /* eTPU QOM API */
#include "etpu_fpm.h"           /* eTPU FPM API */
#include <stdlib.h>

/* User written include files */
#include "fpm_example.h"        /*include application specific defines. */

#define pulse_width_high 0x20
#define pulse_width_low 0x20

union etpu_events_array my_event_array_QOMA[QOMA_array_size] =
    { (pulse_width_low << 1) + FS_ETPU_PIN_HIGH,
    (pulse_width_high << 1) + FS_ETPU_PIN_LOW,
    (pulse_width_low << 1) + FS_ETPU_PIN_HIGH,
    (pulse_width_high << 1) + FS_ETPU_PIN_LOW
};

union etpu_events_array my_event_array_QOMLO[QOMLO_array_size] =
    { (0x001000 << 1) + FS_ETPU_PIN_LOW,
    (0x001000 << 1) + FS_ETPU_PIN_LOW,
    (0x001000 << 1) + FS_ETPU_PIN_LOW,
    (0x001000 << 1) + FS_ETPU_PIN_LOW
};

#define ETPU_INPUT_PAD_CONFIG  FS_GPIO_PRIMARY_FUNCTION + \
                               FS_GPIO_INPUT_BUFFER_ENABLE

#define ETPU_OUTPUT_PAD_CONFIG FS_GPIO_PRIMARY_FUNCTION + \
                               FS_GPIO_MAXIMUM_SLEW_RATE + \
                               FS_GPIO_OPEN_DRAIN_DISABLE + \
                               FS_GPIO_INPUT_BUFFER_ENABLE + \
                               FS_GPIO_OUTPUT_BUFFER_ENABLE

#define N_SAMPLES 200

uint32_t *fs_free_param;
const uint32_t etpu_a_tcr1_freq = 64000000;     /* 64MHz */

void delay (vint32_t random);

main ()
{
    uint32_t freq[12];
    int32_t error_code;
    vuint16_t *upper_QOM_SCR16; /* pointers to eTPU QOM SCR */
    vuint32_t *upper_QOM_SCR32;
    vuint16_t *upper_FPM_SCR16; /* pointers to eTPU FPM SCR */
    vuint32_t *upper_FPM_SCR32;
    uint32_t *QOM_ram_ptr;      /**pointer to QOM ram */
    uint32_t *QOM_last_match_ptr;
    uint32_t tcr1_now;          /*store for read tcr value */
    int32_t n;
    uint32_t supplied_freq;
    uint32_t count[12];
    int8_t count_ok;

    upper_QOM_SCR32 = &ETPU.CHAN[QOM0].SCR.R;
    upper_QOM_SCR16 = (vuint16_t *) upper_QOM_SCR32;
    upper_FPM_SCR32 = &ETPU.CHAN[FPM0].SCR.R;
    upper_FPM_SCR16 = (vuint16_t *) upper_FPM_SCR32;

    /* initialize PLL */
    FMPLL.SYNCR.R = 0x06000000; /* MFD = 0b110, RFD=0b000, 8MHz*16=128MHz */

    /* initialize eTPU hardware */
    fs_etpu_init (my_etpu_config, (uint32_t *) etpu_code,
                  sizeof (etpu_code), (uint32_t *) etpu_globals,
                  sizeof (etpu_globals));

    /* enable all timebases */
    fs_timer_start ();

/* configure the pins on the MPC5554*/
/* QOM pin */
    fs_gpio_config (FS_GPIO_ETPUA8, ETPU_OUTPUT_PAD_CONFIG);

/* FPM pin */
    fs_gpio_config (FS_GPIO_ETPUB11, ETPU_INPUT_PAD_CONFIG);

    *upper_QOM_SCR16 = 0xC0C0;  /* clear QOM DMA/interrupt service request */
    *upper_FPM_SCR16 = 0xC0C0;  /* clear FPM DMA/interrupt service request */

/************************************************************/
/*                                                          */
/*                                                          */
/* start of test4: simple 1MHz rising edge test; continuous */
/*                                                          */
/*                                                          */
/************************************************************/

    /* init QOM channel Low using QOM fn */
    error_code =
        fs_etpu_qom_init (QOM0,
                          FS_ETPU_PRIORITY_MIDDLE,
                          FS_ETPU_QOM_SINGLE_SHOT,
                          FS_ETPU_TCR1,
                          FS_ETPU_QOM_INIT_PIN_LOW,
                          FS_ETPU_QOM_IMMEDIATE,
                          (uint32_t *) 0,
                          7, QOMLO_array_size, my_event_array_QOMLO);

    while (ETPU.CHAN[QOM0].SCR.B.CIS == 0)
      {
      };
    /*wait for QOM to finish */

    *upper_QOM_SCR16 = 0xC0C0;
    /* clear DMA/interrupt service request */

    QOM_ram_ptr = fs_etpu_data_ram (QOM0);
    QOM_last_match_ptr =
        (uint32_t *) ((uint32_t) QOM_ram_ptr +
                      FS_ETPU_QOM_LAST_MATCH_TIME_OFFSET - 1);
    /* get address to be used as reference for the QOM fn */

    /*initialise the channel for FPM */

    error_code =
        fs_etpu_fpm_init (FPM0,
                          FS_ETPU_PRIORITY_MIDDLE,
                          FS_ETPU_FPM_CONTINUOUS,
                          FS_ETPU_FPM_RISING_EDGE, 
                          FS_ETPU_TCR1, 0x700);

    *upper_FPM_SCR16 = 0xC0C0;
    /* clear DMA/interrupt service request */
    /* this interrupt is a hangover from last
       test */

    tcr1_now = ETPU.TB1R_A.R;

    *QOM_last_match_ptr = tcr1_now + 0x4000;
    /* program reference for QOM fn */

/*initialise the channel for QOM */
    error_code =
        fs_etpu_qom_init (QOM0,
                          FS_ETPU_PRIORITY_MIDDLE,
                          FS_ETPU_QOM_CONTINUOUS,
                          FS_ETPU_TCR1,
                          FS_ETPU_QOM_INIT_PIN_LOW,
                          FS_ETPU_QOM_USE_REF_ADDRESS,
                          (uint32_t *) QOM_last_match_ptr,
                          12, 
                          QOMA_array_size, 
                          my_event_array_QOMA);

    if (error_code != 0)
        return (QOM0 + 1);

    supplied_freq = etpu_a_tcr1_freq / (pulse_width_low + pulse_width_high);

    for (n = 0; n < 12; n++)
      {
          while (ETPU.CHAN[FPM0].SCR.B.CIS == 0)
            {
            };
          /*wait for FPM0 to finish */

          *upper_FPM_SCR16 = 0xC0C0;
          /* clear DMA/interrupt service request */

          freq[n] = fs_etpu_fpm_get_freq (FPM0, etpu_a_tcr1_freq);
          count[n] = fs_etpu_fpm_get_count (FPM0);

        test4:
          while (freq[n] != supplied_freq)
            {
            }
          count_ok = 0;
          if (count[0] == 27 || count[n] == 28)
            {
                count_ok = 1;
            }
          while (count_ok != 1)
            {
            }
      }

/************************************************************/
/*                                                          */
/*                                                          */
/* end of test4: simple 1MHz rising edge test; continuous   */
/*                                                          */
/*                                                          */
/************************************************************/

  idle:
    for (;;)
      {
      }                         /* All done. */
}

/*********************************************************************
 *
 * Copyright:
 *	FREESCALE, INC. All Rights Reserved.
 *  You are hereby granted a copyright license to use, modify, and
 *  distribute the SOFTWARE so long as this entire notice is
 *  retained without alteration in any modified and/or redistributed
 *  versions, and that such modified versions are clearly identified
 *  as such. No licenses are granted by implication, estoppel or
 *  otherwise under any patents or trademarks of Motorola, Inc. This
 *  software is provided on an "AS IS" basis and without warranty.
 *
 *  To the maximum extent permitted by applicable law, FREESCALE
 *  DISCLAIMS ALL WARRANTIES WHETHER EXPRESS OR IMPLIED, INCLUDING
 *  IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR
 *  PURPOSE AND ANY WARRANTY AGAINST INFRINGEMENT WITH REGARD TO THE
 *  SOFTWARE (INCLUDING ANY MODIFIED VERSIONS THEREOF) AND ANY
 *  ACCOMPANYING WRITTEN MATERIALS.
 *
 *  To the maximum extent permitted by applicable law, IN NO EVENT
 *  SHALL FREESCALE BE LIABLE FOR ANY DAMAGES WHATSOEVER (INCLUDING
 *  WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS
 *  INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY
 *  LOSS) ARISING OF THE USE OR INABILITY TO USE THE SOFTWARE.
 *
 *  Freescale assumes no responsibility for the maintenance and support
 *  of this software
 ********************************************************************/
